import { addMocksToSchema } from '@graphql-tools/mock';
import { MockLink } from '@apollo/client/testing';
import { ApolloClient, ApolloLink, InMemoryCache, Observable } from '@apollo/client';
import { buildClientSchema, graphql, print, GraphQLError } from 'graphql';
function delay(ms) {
    return new Promise((resolve)=>{
        setTimeout(()=>{
            resolve();
        }, ms);
    });
}
function createMockLink(schema, rootValue = {}, context = {}, options = {}) {
    const delayMs = options?.delay ?? 0;
    return new ApolloLink((operation)=>{
        return new Observable((observer)=>{
            const { query, operationName, variables } = operation;
            delay(delayMs).then(()=>{
                return graphql({
                    schema,
                    source: print(query),
                    rootValue,
                    contextValue: context,
                    variableValues: variables,
                    operationName
                });
            }).then((result)=>{
                const originalError = result?.errors?.[0].originalError;
                if (originalError) {
                    // @ts-ignore
                    const graphQLErrors = originalError?.graphQLErrors;
                    if (graphQLErrors) {
                        observer.next({
                            errors: graphQLErrors
                        });
                    }
                    // @ts-ignore
                    const networkError = originalError.networkError;
                    if (networkError) {
                        observer.error(networkError);
                    }
                } else {
                    observer.next(result);
                }
                observer.complete();
            }).catch(observer.error.bind(observer));
        });
    });
}
export function createErrorLink(graphQLError) {
    return new ApolloLink(()=>{
        return new Observable((observer)=>{
            delay(100).then(()=>{
                observer.next({
                    errors: createGraphQLErrorMessage(graphQLError)
                });
                observer.complete();
            }).catch(observer.error.bind(observer));
        });
    });
}
export function createLoadingLink() {
    return new ApolloLink(()=>{
        // eslint-disable-next-line @typescript-eslint/no-empty-function
        return new Observable(()=>{});
    });
}
export function createApolloClient({ mocks, delay = 0, cacheOptions = {}, clientOptions = {}, links = ()=>{
    return [];
}, addTypename = false }) {
    let mockLink;
    if (!Array.isArray(mocks)) {
        const apolloLinkOptions = {};
        const { resolvers, introspectionResult, rootValue, context } = mocks;
        const schema = buildClientSchema(introspectionResult);
        // @ts-ignore
        const schemaWithMocks = addMocksToSchema({
            schema,
            resolvers
        });
        if (delay) {
            apolloLinkOptions.delay = delay;
        }
        mockLink = createMockLink(schemaWithMocks, rootValue, context, apolloLinkOptions);
    } else {
        mockLink = new MockLink(mocks, addTypename);
    }
    const cache = new InMemoryCache({
        ...cacheOptions,
        addTypename
    });
    return new ApolloClient({
        // @ts-ignore
        cache,
        link: ApolloLink.from([
            ...links(cache),
            mockLink
        ]),
        ...clientOptions
    });
}
export function createGraphQLErrorMessage(graphQLError) {
    if (graphQLError) {
        return typeof graphQLError === 'string' ? [
            new GraphQLError(graphQLError)
        ] : graphQLError;
    }
    return [
        new GraphQLError('Unspecified error from ErrorProvider.')
    ];
}
export function createMocks(query, { variables, data, newData, graphqlErrors, error, delay = 0 }) {
    return [
        {
            request: {
                query,
                variables
            },
            ...!error ? {
                result: {
                    data,
                    ...graphqlErrors ? {
                        errors: createGraphQLErrorMessage(graphqlErrors)
                    } : {}
                }
            } : {},
            error,
            newData,
            delay
        }
    ];
}

//# sourceMappingURL=utils.js.map